local MAJOR, MINOR = "Xii_LibBit-1.0", 1
local lib = LibStub:NewLibrary(MAJOR, MINOR)


local bit = bit32 or bit
local bor, band, bxor, bnot, brshift, blshift = bit.bor, bit.band, bit.bxor, bit.bnot, bit.rshift, bit.lshift
local ipars, pairs, unpack, tinsert = ipairs, pairs, unpack, table.insert


if not lib then
  return	-- already loaded and no upgrade necessary
end

-- Mixin
for k,v in pairs(bit) do
	lib[k] = v
end

--[[
	http://bitop.luajit.org/api.html

	bit.tobit
	bit.tohex
	bit.bnot
	bit.band
	bit.bor
	bit.bxor
	bit.lshift
	bit.rshift
	bit.arshift
	bit.rol
	bit.ror
	bit.bswap
]]

local t32 = {
	    [0] = 0;  9,  1, 10, 13, 21,  2, 29,
	    11, 14, 16, 18, 22, 25,  3, 30,
	    8, 12, 20, 28, 15, 17, 24,  7,
	    19, 27, 23,  6, 26,  5,  4, 31 }


function lib.logBase2(v)
	v = bor(v, brshift(v, 1))
	v = bor(v, brshift(v, 2))
	v = bor(v, brshift(v, 4))
	v = bor(v, brshift(v, 8))
	v = bor(v, brshift(v, 16))

	return t32[brshift(v*0x07c4acdd, 27)]
end


function lib.hasAnyOf(state, ...)
	local states = bor(...)
	return (band(state, states) ~= 0)
end


function lib.hasAllOf(state, ...)
	local states = bor(...)
	return (band(state, states) == states)
end


function lib.hasNoneOf(state, ...)
	return not lib.hasAnyOf(state, ...)
end


function lib.hasXor(state, ...)
	local results = {}

	for _,flag in ipairs{...} do
		tinsert(results, band(state, flag) == flag and 1 or 0)
	end

	return bxor(unpack(results))
end


function lib.toggle(...)
	return bxor(...)
end


function lib.unset(state, ...)
	return band(state, bnot(bor(...)))
end


function lib.findLogBase2(v)
	-- local v  -- 32-bit value to find the log2 of 
	local b = {0x2, 0xC, 0xF0, 0xFF00, 0xFFFF0000}
	local S = {1, 2, 4, 8, 16}

	local r = 0 -- result of log2(v) will go here
	
	for i = 5, 1, -1 do -- unroll for speed...
	 	if (band(v, b[i]) ~= 0) then
			v = brshift(v, S[i])
			r = bor(r, S[i])
		end 
	end

	return r
end



-- Alias for bit.bor
lib.set = bor 

return lib
